<?php

declare(strict_types=1);

namespace spec\Dedi\SyliusSogenactifPlugin\Payum\Sogenactif;

use Dedi\SyliusSogenactifPlugin\Payum\Sogenactif\SealEncoderInterface;
use PhpSpec\ObjectBehavior;
use Webmozart\Assert\Assert;

final class SealEncoderSpec extends ObjectBehavior
{
    public function getMatchers(): array
    {
        return [
            'encodedAsUtf8' => function ($subject) {
                return mb_detect_encoding($subject, 'UTF-8', true);
            },
            'hashEquals' => function ($subject, $hashToCompareWith) {
                return hash_equals($subject, $hashToCompareWith);
            },
            'hashNotEquals' => function ($subject, $hashToCompareWith) {
                return !hash_equals($subject, $hashToCompareWith);
            },
        ];
    }
    public function it_should_be_an_encoder()
    {
        $this->shouldImplement(SealEncoderInterface::class);
    }

    public function it_should_return_encodeding_value()
    {
        $return = $this->encodeUtf8("");

        $return->shouldBeString();
        $return->shouldReturn("");
        $return->shouldEncodedAsUtf8();

        $return = $this->encodeUtf8("foo");

        $return->shouldBeString();
        $return->shouldReturn("foo");
        $return->shouldEncodedAsUtf8();

        $return = $this->encodeUtf8("bar");

        $return->shouldBeString();

        $return->shouldReturn("bar");
        $return->shouldEncodedAsUtf8();
    }

    public function it_should_return_no_data_with_empty_array()
    {
        $return = $this->createSeal([], "");

        $return->shouldBeString();
        $return->shouldNotEqual("");
    }

    public function it_should_encoded()
    {
        $this->createSeal([], "");
    }

    public function it_should_concate_string_with_equel_glue()
    {
        $this
            ->concateStrineWith(['foo', 'bar'])
            ->shouldReturn('foobar')
        ;
    }

    public function it_should_return_a_non_empty_value_as_data_array_are_given()
    {
        $return = $this->createSeal([
            'foo' => 'bar',
        ], "");

        $return->shouldNotBe("");
    }

    public function it_should_call_encoded_function()
    {
        $return = $this->createSeal([
            'foo' => 'bar',
        ], "");

        $return->shouldNotBe("");
    }

    public function it_should_hash()
    {
        $value = $this->hashWithAlgo('ripemd160', 'Le rapide goupil brun sauta par dessus le chien paresseux.', 'secret');
        $value->shouldBeString();

        $value->shouldHashEquals("a6f41314c7a3482836268cf504b066e08216e40f");
        $value->shouldHashNotEquals("111111111111111111111111111111111111");
    }

    public function it_shoutl_return_well_concate_string()
    {
        // Base on exemple from doc but int the given result their is no "keyVersion": "1" in the encoded chain from the doc
        $result = $this->concateStrineWith(json_decode('{
              "amount": "2500",
              "automaticResponseUrl": "https://automatic-response-url.fr/",
              "captureDay": "0",
              "captureMode": "AUTHOR_CAPTURE",
              "currencyCode": "978",
              "interfaceVersion": "IR_WS_2.22",
              "merchantId": "011223344550000",
              "normalReturnUrl": "https://normal-return-url/",
              "orderChannel": "INTERNET",
              "orderId": "ORD101",
              "returnContext": "ReturnContext",
              "transactionOrigin": "SO_WEBAPPLI",
              "transactionReference": "TREFEXA2012"
            }', true));


        $result->shouldBeString();
        $result->shouldBe("2500https://automatic-response-url.fr/0AUTHOR_CAPTURE978IR_WS_2.22011223344550000https://normal-return-url/INTERNETORD101ReturnContextSO_WEBAPPLITREFEXA2012");

        // Base on exemple from doc but int the given result their is no "keyVersion": "1" in the encoded chain from the doc
        $result = $this->concateStrineWith(json_decode(<<<'BAR'
{
    "amount": "2500",
    "captureDay": "0",
    "captureMode": "AUTHOR_CAPTURE",
    "cardCSCValue": "000",
    "cardExpiryDate": "201612",
    "cardNumber": "1234123412341234",
    "currencyCode": "978",
    "interfaceVersion": "IR_WS_2.3",
    "merchantId": "011223344550000",
    "orderChannel": "INTERNET",
    "orderId": "ORD101",
    "returnContext": "ReturnContext",
    "transactionOrigin": "SO_WEBAPPLI",
    "transactionReference": "TREFEXA2012"
}
BAR
, true));


            $result->shouldBeString();
            $result->shouldBe("25000AUTHOR_CAPTURE0002016121234123412341234978IR_WS_2.3011223344550000INTERNETORD101ReturnContextSO_WEBAPPLITREFEXA2012");
        }


        public function it_should_return_value_concate_well_encoded()
        {
            // Base on exemple from doc but int the given result their is no "keyVersion": "1" in the encoded chain from the doc
            $result = $this->createSeal(json_decode(<<<'FOO'
{
  "amount": "2500",
  "automaticResponseUrl": "https://automatic-response-url.fr/",
  "captureDay": "0",
  "captureMode": "AUTHOR_CAPTURE",
  "currencyCode": "978",
  "interfaceVersion": "IR_WS_2.22",
  "merchantId": "011223344550000",
  "normalReturnUrl": "https://normal-return-url/",
  "orderChannel": "INTERNET",
  "orderId": "ORD101",
  "returnContext": "ReturnContext",
  "transactionOrigin": "SO_WEBAPPLI",
  "transactionReference": "TREFEXA2012"
}
FOO
, true), 'secret123');

        $result->shouldBeString();
        $result->shouldBe('e4d75c6a779adee5850192d1ecc162ca0c85e960cb00d9923879609f89739152');

//        // Base on exemple from doc but int the given result their is no "keyVersion": "1" in the encoded chain from the doc
        $result = $this->createSeal(json_decode(<<<'FOO'
{
  "amount": "2500",
  "captureDay": "0",
  "captureMode": "AUTHOR_CAPTURE",
  "cardCSCValue": "000",
  "cardExpiryDate": "201612",
  "cardNumber": "1234123412341234",
  "currencyCode": "978",
  "interfaceVersion": "IR_WS_2.3",
  "merchantId": "011223344550000",
  "orderChannel": "INTERNET",
  "orderId": "ORD101",
  "returnContext": "ReturnContext",
  "transactionOrigin": "SO_WEBAPPLI",
  "transactionReference": "TREFEXA2012"
}
FOO
, true), 'secret123');

        $result->shouldBeString();
        $result->shouldBe('b434da570ba8bc677982e4d76f7292d18c571689f6552a7048653d2a844482b3');
//        // Base on exemple from doc but int the given result their is no "keyVersion": "1" in the encoded chain from the doc
        $result = $this->createSeal(json_decode(<<<'FOO'
{
  "amount": "2500",
  "captureDay": "0",
  "captureMode": "AUTHOR_CAPTURE",
  "cardCSCValue": "000",
  "cardExpiryDate": "201612",
  "cardNumber": "1234123412341234",
  "currencyCode": "978",
  "interfaceVersion": "IR_WS_2.3",
  "merchantId": "011223344550000",
  "orderChannel": "INTERNET",
  "orderId": "ORD101",
  "returnContext": "ReturnContext",
  "transactionOrigin": "SO_WEBAPPLI",
  "transactionReference": "TREFEXA2012"
}
FOO
, true), 'secret123');

        $result->shouldBeString();
        $result->shouldBe('b434da570ba8bc677982e4d76f7292d18c571689f6552a7048653d2a844482b3');

        $result->shouldBeString();
        $result->shouldBe('b434da570ba8bc677982e4d76f7292d18c571689f6552a7048653d2a844482b3');

        $result = $this->createSeal(json_decode(<<<'FOO'
{
    "amount": "1000",
    "automaticResponseUrl" : "https://responseurl.com",
    "currencyCode":"978",
    "interfaceVersion" : "IR_WS_2.3",
    "merchantId": "002010000000002",
    "normalReturnUrl" : "https://responseurl2.com",
    "orderChannel" : "INTERNET",
    "transactionReference" : "1232015021717315"
}
FOO
, true), '002010000000002_KEY1');

        $result->shouldBeString();

    }
}
